function [ area, NBlobs, BlobCenter ] = Counting( Dish, DishBW, TotArea)
% COUNTING estimate number of colonies in a Petri dish
%   Counting takes the image of the Petri dish with (yeast) colonies
%   and estimates the number of colonies.
%   The initial estimate is based on the shape (roundness) of colonies.
%   Then, the objects containing several colonies, touching each other,
%   are resolved using circular correlation mask.
%   Several filters are used to reduce image noise,
%   to separate colonies from each other as much as possible
%   and to reduce inaccuracy of the counting result.
%
%   Inputs:
%   Dish, DishBW - greyscale and B/W image of the colonies
%   R_ROI - radius of the region-of-interest (ROI)
%   TotArea - total area of ROI
%
% Author Jan Schier <schier@utia.cas.cz>
% Date 2009/08/04
%
% Copyright
% (c) 2009 Dept. of Signal Processing
% Institute of Information Theory and Automation
% Academy of Sciences of the Czech Republic, Prague
%
% http://sp.utia.cz
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 2 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% You are free to distribute this software under Version 2.1
% of the GNU General Public License.

DishEroded=zeros(size(Dish)); % Eroded image

ColonyArea = sum(sum(DishBW));
% Relative area in percent, 2 decimal points
area = round(ColonyArea/TotArea*10000)/100;

% Filter image
Colonies = Dish.*DishBW; % Filter colonies from the original image
Mask = strel('diamond',1); % Erosion mask
% Erode image to filter out noise speckles
% Leave 1 pixel frame around, for easier indexing later
DishEroded(2:end-1,2:end-1) = imerode(DishBW(2:end-1,2:end-1),Mask);
DishEroded = imerode(DishEroded,Mask);

% The edge of the colonies is darker than the center,
% by setting the threshold between the intensity of the edge and of the
% center, we can discrimate some colonies that touch each other
ColoniesEdge = Colonies;
ColoniesEdge(imerode(DishEroded,Mask)>0)=0;
% Histogram of the edge
[Hist,xout]=hist(ColoniesEdge(ColoniesEdge>0));
[HistMax,IdxMax]=max(Hist);
% Maximum of the histogram
THold=xout(IdxMax);

% Binary image of the center part of colonies
ImgROI = Colonies;
ImgROI(ImgROI<THold) = 0;
ImgROI(ImgROI>0) = 1;
ImgROI = imfill(ImgROI,'holes');
ImgROI = imerode(ImgROI,Mask);
ImgROI = imerode(ImgROI,Mask);


% Find round, isolated colonies

% Label and cound all blobs in the bw image
[DiamLabel, NBlobs] = bwlabeln(ImgROI);
% Inspect properties of the blobs
Properties = regionprops(DiamLabel, 'EquivDiameter', 'Centroid', 'BoundingBox', 'Eccentricity');
BlobCenter = zeros(length(Properties),2);
OutlierIdx=[];

for idx=1:length(Properties)
    BlobCenter(idx,2) = round(Properties(idx).Centroid(1));
    BlobCenter(idx,1) = round(Properties(idx).Centroid(2));
    BlobRadius(idx) = round(Properties(idx).EquivDiameter/2);
    Eccentr(idx) = Properties(idx).Eccentricity;
    % Detected center is located outside the blob pixels
    if (ImgROI(BlobCenter(idx,1),BlobCenter(idx,2))==0)
        OutlierIdx(end+1) = idx;
    end
end

% Only blobs with eccentricity<=0.8 are considered (roughly) round
% blobs with higher eccentricity are assumed two or more colonies touching
% each other
Outlier = find(Eccentr>0.8);
OutlierIdx = [OutlierIdx Outlier];
DishOutliers = zeros(size(ImgROI));
for idx=1:length(OutlierIdx)
    BlobCenter(OutlierIdx(idx),1)=NaN;
    BlobCenter(OutlierIdx(idx),2)=NaN;
    BlobRadius(OutlierIdx(idx))=NaN;
    DishOutliers(DiamLabel==OutlierIdx(idx))=1;
end

% Subtract the blobs rejected above from the total number of blobs
NumberBlobs = NBlobs-length(OutlierIdx);

% From the blobs with round shape, estimate blob radius
% 'mean' could be probably used as well
BlobEqRadius = round(mean(BlobRadius(isfinite(BlobRadius))));
BlobRadMin = BlobEqRadius-1;
BlobRadMax = BlobEqRadius+1;
BlobDiameter = 2*BlobRadMax+1;
NBlobPixels = (BlobRadMin*2)^2;

if NumberBlobs>0
    
    %% Correlation mask for blob counting
    
    
    BlobMask = CircleSeg45(BlobRadMax, 0, 0);
    tmp = CircleSeg45(BlobRadMax, 0, 1);
    BlobMask = [BlobMask zeros(1,BlobRadMax-length(BlobMask))];
    BlobMask(end:-1:BlobRadMax-length(tmp)+1) = tmp;
    
    % Prepare matrix containing only region of interest
    CorrMask  =zeros(BlobDiameter,BlobDiameter); % BW image
    CorrMask(BlobRadMax+1,:) = ones(1,BlobDiameter);
    for i=1:BlobRadMax
        CorrMask(BlobRadMax+1-i,BlobRadMax+1+(-BlobMask(i):BlobMask(i))) = ...
            ones(1,2*BlobMask(i)+1);
        CorrMask(BlobRadMax+1+i,BlobRadMax+1+(-BlobMask(i):BlobMask(i))) = ...
            ones(1,2*BlobMask(i)+1);
    end
    
    BlobMask = CircleSeg45(BlobRadMin, 0, 0);
    tmp = CircleSeg45(BlobRadMin, 0, 1);
    BlobMask = [BlobMask zeros(1,BlobRadMin-length(BlobMask))];
    BlobMask(end:-1:BlobRadMin-length(tmp)+1) = tmp;
    
    %% Prepare matrix containing only region of interest
    CorrMask(BlobRadMax+1,BlobRadMax+1+(-(BlobRadMin):(BlobRadMin))) = zeros(1,2*BlobRadMin+1);
    for i=1:BlobRadMin
        CorrMask(BlobRadMax+1-i,BlobRadMax+1+(-BlobMask(i):BlobMask(i))) = ...
            zeros(1,2*BlobMask(i)+1);
        CorrMask(BlobRadMax+1+i,BlobRadMax+1+(-BlobMask(i):BlobMask(i))) = ...
            zeros(1,2*BlobMask(i)+1);
    end
    
    if NBlobPixels>0
        for idx=1:length(OutlierIdx)
            x1 = round(Properties(OutlierIdx(idx)).BoundingBox(1));
            y1 = round(Properties(OutlierIdx(idx)).BoundingBox(2));
            x2 = x1+round(Properties(OutlierIdx(idx)).BoundingBox(3))-1;
            y2 = y1+round(Properties(OutlierIdx(idx)).BoundingBox(4))-1;
            Region = DishOutliers(y1:y2,x1:x2);
            NPixels = sum(sum(Region));
            RegEroded = imerode(Region,Mask);
            % Region-RegEroded gives only the contour of the image
            ConvRegionMask = conv2(Region-RegEroded,CorrMask,'same').*RegEroded;
            % Only focus on the convolution output
            % BlobRadMax from the 'Region' edge
            Maxima = ConvRegionMask(BlobRadMax:end-BlobRadMax+1,BlobRadMax:end-BlobRadMax+1);
            [sMaxX, sMaxY] = size(Maxima);
            % Rough estimate of the number of colonies
            NBlobsRegion = round(NPixels/(1.2*NBlobPixels));
            if sMaxX>0 & sMaxY>0
                % Find maxima, limit to NBlobsRegion
                for Idx = 1:NBlobsRegion
                    if sMaxX>1
                        [MaxY,Y] = max(Maxima);
                        [MaxX,x] = max(MaxY);
                        y = Y(x);
                        Maxima(max(1,y-BlobRadMin):min(y+BlobRadMin,sMaxX),max(1,x-BlobRadMin):min(x+BlobRadMin,sMaxY))=0;
                    else
                        [MaxX,x] = max(Maxima);
                        y = 1;
                        Maxima(max(1,x-BlobRadMin):min(x+BlobRadMin,sMaxX),max(1,y-BlobRadMin):min(y+BlobRadMin,sMaxY))=0;
                    end
                    BlobCenter(end+1,:) = [y+y1+BlobRadMax; x+x1+BlobRadMax];
                    NBlobs = NBlobs+1;
                end
            end
        end
    end
    
end

end

